@@ -7,6 +7,10 @@ var searchData=
   ['libinput_5fbutton_5fstate',['libinput_button_state',['../group__device.html#ga543a8bb3f2d7421f76e63bb4ee2fb5bf',1,'libinput.h']]],
   ['libinput_5fbutton_5fstate_5fpressed',['LIBINPUT_BUTTON_STATE_PRESSED',['../group__device.html#gga543a8bb3f2d7421f76e63bb4ee2fb5bfa68b58fac43509a92230c315993dcf6b4',1,'libinput.h']]],
   ['libinput_5fbutton_5fstate_5freleased',['LIBINPUT_BUTTON_STATE_RELEASED',['../group__device.html#gga543a8bb3f2d7421f76e63bb4ee2fb5bfad20bd759a424b497d7d363ef136f856b',1,'libinput.h']]],
+  ['libinput_5fconfig_5fclick_5fmethod',['libinput_config_click_method',['../group__config.html#ga3e5dc19a83752430d5e8be7fb2020663',1,'libinput.h']]],
+  ['libinput_5fconfig_5fclick_5fmethod_5fbutton_5fareas',['LIBINPUT_CONFIG_CLICK_METHOD_BUTTON_AREAS',['../group__config.html#gga3e5dc19a83752430d5e8be7fb2020663ab8ce100b5396698712579447b6d2e173',1,'libinput.h']]],
+  ['libinput_5fconfig_5fclick_5fmethod_5fclickfinger',['LIBINPUT_CONFIG_CLICK_METHOD_CLICKFINGER',['../group__config.html#gga3e5dc19a83752430d5e8be7fb2020663af81af7bdec6f0f797abf6d68f294d276',1,'libinput.h']]],
+  ['libinput_5fconfig_5fclick_5fmethod_5fnone',['LIBINPUT_CONFIG_CLICK_METHOD_NONE',['../group__config.html#gga3e5dc19a83752430d5e8be7fb2020663a44f53f91f3b24a6d41ae9b04420ab374',1,'libinput.h']]],
   ['libinput_5fconfig_5fscroll_5f2fg',['LIBINPUT_CONFIG_SCROLL_2FG',['../group__config.html#gga6466e6b9c55e6835a05c7c29c8b7a844aaa2f1aa2d40a3e64129ae4bc3ebac026',1,'libinput.h']]],
   ['libinput_5fconfig_5fscroll_5fedge',['LIBINPUT_CONFIG_SCROLL_EDGE',['../group__config.html#gga6466e6b9c55e6835a05c7c29c8b7a844a294ce3d2e42044573a5eaad2b3cf8983',1,'libinput.h']]],
   ['libinput_5fconfig_5fscroll_5fmethod',['libinput_config_scroll_method',['../group__config.html#ga6466e6b9c55e6835a05c7c29c8b7a844',1,'libinput.h']]],
@@ -37,6 +41,10 @@ var searchData=
   ['libinput_5fdevice_5fconfig_5fcalibration_5fget_5fmatrix',['libinput_device_config_calibration_get_matrix',['../group__config.html#gaa16e8b272ce94f8037822dc8c89d84a5',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fcalibration_5fhas_5fmatrix',['libinput_device_config_calibration_has_matrix',['../group__config.html#gadd921b0caef6fcf05199a93e5ee0728b',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fcalibration_5fset_5fmatrix',['libinput_device_config_calibration_set_matrix',['../group__config.html#ga09a798f58cc601edd2797780096e9804',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5fclick_5fget_5fdefault_5fmethod',['libinput_device_config_click_get_default_method',['../group__config.html#ga574f5bce37a87b6e05a956481748e9b4',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5fclick_5fget_5fmethod',['libinput_device_config_click_get_method',['../group__config.html#ga6aa5fc908b050beb6c0346a09478cad4',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5fclick_5fget_5fmethods',['libinput_device_config_click_get_methods',['../group__config.html#ga30f433d7c21fc8e65dd218079ce1a2a9',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5fclick_5fset_5fmethod',['libinput_device_config_click_set_method',['../group__config.html#gadfd0d6c30f8d87ae570c9159953de3c9',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fleft_5fhanded_5fget',['libinput_device_config_left_handed_get',['../group__config.html#ga998945b1c4f96a9abd3c3a26e5c4166c',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fleft_5fhanded_5fget_5fdefault',['libinput_device_config_left_handed_get_default',['../group__config.html#gaa438a14e32894791e435cb43826ef3a9',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fleft_5fhanded_5fis_5favailable',['libinput_device_config_left_handed_is_available',['../group__config.html#ga755352f2bbb869446aa591dd9d9c5eaf',1,'libinput.h']]],
@@ -61,6 +69,7 @@ var searchData=
   ['libinput_5fdevice_5fconfig_5ftap_5fget_5ffinger_5fcount',['libinput_device_config_tap_get_finger_count',['../group__config.html#ga13973d6bd75a99708723e3c6684517c1',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5ftap_5fset_5fenabled',['libinput_device_config_tap_set_enabled',['../group__config.html#ga3caf860883d4e2e42b7842599da03e56',1,'libinput.h']]],
   ['libinput_5fdevice_5fget_5fcontext',['libinput_device_get_context',['../group__device.html#ga90c6868911d8aba103278c9208d5205e',1,'libinput.h']]],
+  ['libinput_5fdevice_5fget_5fdevice_5fgroup',['libinput_device_get_device_group',['../group__device.html#ga0edc58e5a022b48a8d8416f63d2860df',1,'libinput.h']]],
   ['libinput_5fdevice_5fget_5fid_5fproduct',['libinput_device_get_id_product',['../group__device.html#gae011910b4a673f1293a20a84be4e52e2',1,'libinput.h']]],
   ['libinput_5fdevice_5fget_5fid_5fvendor',['libinput_device_get_id_vendor',['../group__device.html#gad9e4c074b03da790de5e117c71ce8d50',1,'libinput.h']]],
   ['libinput_5fdevice_5fget_5fname',['libinput_device_get_name',['../group__device.html#ga31655d2c2f173d059b3f891801c0a429',1,'libinput.h']]],
@@ -68,11 +77,16 @@ var searchData=
   ['libinput_5fdevice_5fget_5fseat',['libinput_device_get_seat',['../group__device.html#ga27823edb40750992a1a05e7a9f5dc8f0',1,'libinput.h']]],
   ['libinput_5fdevice_5fget_5fsize',['libinput_device_get_size',['../group__device.html#gab91ae9014902fafe4f015e1f42233d27',1,'libinput.h']]],
   ['libinput_5fdevice_5fget_5fsysname',['libinput_device_get_sysname',['../group__device.html#ga106bbe58e6c3a6d1327c9bcbbf2a03f6',1,'libinput.h']]],
-  ['libinput_5fdevice_5fget_5fudev_5fdevice',['libinput_device_get_udev_device',['../libinput_8h.html#aedc690c6679f547c46d9f41182a0e7c9',1,'libinput.h']]],
+  ['libinput_5fdevice_5fget_5fudev_5fdevice',['libinput_device_get_udev_device',['../group__device.html#gaedc690c6679f547c46d9f41182a0e7c9',1,'libinput.h']]],
   ['libinput_5fdevice_5fget_5fuser_5fdata',['libinput_device_get_user_data',['../group__device.html#gaf5c902f56206a5d441e50c3a95ec458b',1,'libinput.h']]],
-  ['libinput_5fdevice_5fhas_5fbutton',['libinput_device_has_button',['../group__device.html#ga780f76b0605494c9f7ee994637d90900',1,'libinput.h']]],
+  ['libinput_5fdevice_5fgroup',['libinput_device_group',['../structlibinput__device__group.html',1,'']]],
+  ['libinput_5fdevice_5fgroup_5fget_5fuser_5fdata',['libinput_device_group_get_user_data',['../group__device.html#ga779d66e502b17a4c4aae6d3c773470f9',1,'libinput.h']]],
+  ['libinput_5fdevice_5fgroup_5fref',['libinput_device_group_ref',['../group__device.html#ga72f2803f681e074e56e0da63e4ccead2',1,'libinput.h']]],
+  ['libinput_5fdevice_5fgroup_5fset_5fuser_5fdata',['libinput_device_group_set_user_data',['../group__device.html#gaf252f7535939d813ebe6eb7b2b792446',1,'libinput.h']]],
+  ['libinput_5fdevice_5fgroup_5funref',['libinput_device_group_unref',['../group__device.html#ga371124d3d3bec0a7f946acd1c4417efe',1,'libinput.h']]],
   ['libinput_5fdevice_5fhas_5fcapability',['libinput_device_has_capability',['../group__device.html#gaa32d887454e8baf8aaeeb635329aef89',1,'libinput.h']]],
   ['libinput_5fdevice_5fled_5fupdate',['libinput_device_led_update',['../group__device.html#gafdb3ddfb91e6bd38b6356a5bb51547ad',1,'libinput.h']]],
+  ['libinput_5fdevice_5fpointer_5fhas_5fbutton',['libinput_device_pointer_has_button',['../group__device.html#gaaf63f10e5120daf174bd7943169f45bf',1,'libinput.h']]],
   ['libinput_5fdevice_5fref',['libinput_device_ref',['../group__device.html#ga3335d55a50f0c2fb0c5f496a3105d467',1,'libinput.h']]],
   ['libinput_5fdevice_5fset_5fseat_5flogical_5fname',['libinput_device_set_seat_logical_name',['../group__device.html#ga9f76acc43c8684a13f232073ee829b50',1,'libinput.h']]],
   ['libinput_5fdevice_5fset_5fuser_5fdata',['libinput_device_set_user_data',['../group__device.html#ga6243ad6005b2ce2a5ecfd1b04787fe93',1,'libinput.h']]],
@@ -148,12 +162,12 @@ var searchData=
   ['libinput_5fled_5fnum_5flock',['LIBINPUT_LED_NUM_LOCK',['../group__device.html#gga37ec9afaec2e5407e3c5248faedd1970aa82d6090fb4615d2379f59c06c79588a',1,'libinput.h']]],
   ['libinput_5fled_5fscroll_5flock',['LIBINPUT_LED_SCROLL_LOCK',['../group__device.html#gga37ec9afaec2e5407e3c5248faedd1970a1914ddccb1739d77543767d7768e3abd',1,'libinput.h']]],
   ['libinput_5flog_5fget_5fpriority',['libinput_log_get_priority',['../group__base.html#ga7367998ca39a6e873920ba5e2a12d4f0',1,'libinput.h']]],
-  ['libinput_5flog_5fhandler',['libinput_log_handler',['../group__base.html#ga387f6d09ed8548128a68f1d19ea675c2',1,'libinput.h']]],
+  ['libinput_5flog_5fhandler',['libinput_log_handler',['../group__base.html#gaa16223d9fa8b6e3ac7c722e184f2217f',1,'libinput.h']]],
   ['libinput_5flog_5fpriority',['libinput_log_priority',['../libinput_8h.html#aa7e9ebbd1a13fe2ed6f44e69df3216e1',1,'libinput.h']]],
   ['libinput_5flog_5fpriority_5fdebug',['LIBINPUT_LOG_PRIORITY_DEBUG',['../libinput_8h.html#aa7e9ebbd1a13fe2ed6f44e69df3216e1a3f46e5b7430e100479b4ea27f0fbb1f8',1,'libinput.h']]],
   ['libinput_5flog_5fpriority_5ferror',['LIBINPUT_LOG_PRIORITY_ERROR',['../libinput_8h.html#aa7e9ebbd1a13fe2ed6f44e69df3216e1ad79f66fcf1d6cbd03678cfc5b012b83d',1,'libinput.h']]],
   ['libinput_5flog_5fpriority_5finfo',['LIBINPUT_LOG_PRIORITY_INFO',['../libinput_8h.html#aa7e9ebbd1a13fe2ed6f44e69df3216e1a9b457be58d6cf5d66ea177aa3d2c5500',1,'libinput.h']]],
-  ['libinput_5flog_5fset_5fhandler',['libinput_log_set_handler',['../group__base.html#gae1e6fac5e7d48904140eeb467d2ca9f2',1,'libinput.h']]],
+  ['libinput_5flog_5fset_5fhandler',['libinput_log_set_handler',['../group__base.html#gacf00228a43c831817ba6ec0b94b9df74',1,'libinput.h']]],
   ['libinput_5flog_5fset_5fpriority',['libinput_log_set_priority',['../group__base.html#ga75a500c54c61cfcecd4c76dbb8022933',1,'libinput.h']]],
   ['libinput_5fnext_5fevent_5ftype',['libinput_next_event_type',['../group__base.html#gac86ad3c0c5bc27d8f630264f46cdf005',1,'libinput.h']]],
   ['libinput_5fpath_5fadd_5fdevice',['libinput_path_add_device',['../group__base.html#gaa797496f0150b482a4e01376bd33a47b',1,'libinput.h']]],
