@@ -80,14 +80,14 @@ button_event_to_str(enum button_event event) {
 static inline bool
 is_inside_bottom_button_area(struct tp_dispatch *tp, struct tp_touch *t)
 {
-	return t->y >= tp->buttons.bottom_area.top_edge;
+	return t->point.y >= tp->buttons.bottom_area.top_edge;
 }
 
 static inline bool
 is_inside_bottom_right_area(struct tp_dispatch *tp, struct tp_touch *t)
 {
 	return is_inside_bottom_button_area(tp, t) &&
-	       t->x > tp->buttons.bottom_area.rightbutton_left_edge;
+	       t->point.x > tp->buttons.bottom_area.rightbutton_left_edge;
 }
 
 static inline bool
@@ -100,29 +100,29 @@ is_inside_bottom_left_area(struct tp_dispatch *tp, struct tp_touch *t)
 static inline bool
 is_inside_top_button_area(struct tp_dispatch *tp, struct tp_touch *t)
 {
-	return t->y <= tp->buttons.top_area.bottom_edge;
+	return t->point.y <= tp->buttons.top_area.bottom_edge;
 }
 
 static inline bool
 is_inside_top_right_area(struct tp_dispatch *tp, struct tp_touch *t)
 {
 	return is_inside_top_button_area(tp, t) &&
-	       t->x > tp->buttons.top_area.rightbutton_left_edge;
+	       t->point.x > tp->buttons.top_area.rightbutton_left_edge;
 }
 
 static inline bool
 is_inside_top_left_area(struct tp_dispatch *tp, struct tp_touch *t)
 {
 	return is_inside_top_button_area(tp, t) &&
-	       t->x < tp->buttons.top_area.leftbutton_right_edge;
+	       t->point.x < tp->buttons.top_area.leftbutton_right_edge;
 }
 
 static inline bool
 is_inside_top_middle_area(struct tp_dispatch *tp, struct tp_touch *t)
 {
 	return is_inside_top_button_area(tp, t) &&
-	       t->x >= tp->buttons.top_area.leftbutton_right_edge &&
-	       t->x <= tp->buttons.top_area.rightbutton_left_edge;
+	       t->point.x >= tp->buttons.top_area.leftbutton_right_edge &&
+	       t->point.x <= tp->buttons.top_area.rightbutton_left_edge;
 }
 
 static void
@@ -156,7 +156,6 @@ tp_button_set_state(struct tp_dispatch *tp, struct tp_touch *t,
 		break;
 	case BUTTON_STATE_AREA:
 		t->button.curr = BUTTON_EVENT_IN_AREA;
-		tp_set_pointer(tp, t);
 		break;
 	case BUTTON_STATE_BOTTOM:
 		t->button.curr = event;
@@ -514,7 +513,7 @@ tp_init_softbuttons(struct tp_dispatch *tp,
 
 	/* button height: 10mm or 15% of the touchpad height,
 	   whichever is smaller */
-	if (yres > 1 && (height * 0.15/yres) > 10) {
+	if (!device->abs.fake_resolution && (height * 0.15/yres) > 10) {
 		tp->buttons.bottom_area.top_edge =
 		absinfo_y->maximum - 10 * yres;
 	} else {
@@ -551,7 +550,7 @@ tp_init_top_softbuttons(struct tp_dispatch *tp,
 		double topsize_mm = 10 * topbutton_size_mult;
 		double topsize_pct = .15 * topbutton_size_mult;
 
-		if (yres > 1) {
+		if (!device->abs.fake_resolution) {
 			tp->buttons.top_area.bottom_edge =
 			yoffset + topsize_mm * yres;
 		} else {
@@ -668,8 +667,8 @@ tp_init_buttons(struct tp_dispatch *tp,
 			log_bug_kernel(libinput,
 				       "%s: clickpad advertising right button\n",
 				       device->devname);
-	} else {
-		if (!tp->buttons.is_clickpad)
+	} else if (libevdev_has_event_code(device->evdev, EV_KEY, BTN_LEFT) &&
+		   !tp->buttons.is_clickpad) {
 			log_bug_kernel(libinput,
 				       "%s: non clickpad without right button?\n",
 				       device->devname);
