@@ -32,6 +32,21 @@
 
 struct libinput_source;
 
+/* A coordinate pair in device coordinates */
+struct device_coords {
+	int x, y;
+};
+
+/* A dpi-normalized coordinate pair */
+struct normalized_coords {
+	double x, y;
+};
+
+/* A discrete step pair (mouse wheels) */
+struct discrete_coords {
+	int x, y;
+};
+
 struct libinput_interface_backend {
 	int (*resume)(struct libinput *libinput);
 	void (*suspend)(struct libinput *libinput);
@@ -165,8 +180,15 @@ struct libinput_device_config {
 	struct libinput_device_config_click_method *click_method;
 };
 
+struct libinput_device_group {
+	int refcount;
+	void *user_data;
+	char *identifier; /* unique identifier or NULL for singletons */
+};
+
 struct libinput_device {
 	struct libinput_seat *seat;
+	struct libinput_device_group *group;
 	struct list link;
 	struct list event_listeners;
 	void *user_data;
@@ -187,7 +209,6 @@ struct libinput_event_listener {
 
 typedef void (*libinput_source_dispatch_t)(void *data);
 
-
 #define log_debug(li_, ...) log_msg((li_), LIBINPUT_LOG_PRIORITY_DEBUG, __VA_ARGS__)
 #define log_info(li_, ...) log_msg((li_), LIBINPUT_LOG_PRIORITY_INFO, __VA_ARGS__)
 #define log_error(li_, ...) log_msg((li_), LIBINPUT_LOG_PRIORITY_ERROR, __VA_ARGS__)
@@ -240,6 +261,13 @@ void
 libinput_device_init(struct libinput_device *device,
 		     struct libinput_seat *seat);
 
+struct libinput_device_group *
+libinput_device_group_create(const char *identifier);
+
+void
+libinput_device_set_device_group(struct libinput_device *device,
+				 struct libinput_device_group *group);
+
 void
 libinput_device_add_event_listener(struct libinput_device *device,
 				   struct libinput_event_listener *listener,
@@ -267,16 +295,13 @@ keyboard_notify_key(struct libinput_device *device,
 void
 pointer_notify_motion(struct libinput_device *device,
 		      uint64_t time,
-		      double dx,
-		      double dy,
-		      double dx_noaccel,
-		      double dy_noaccel);
+		      const struct normalized_coords *delta,
+		      const struct normalized_coords *unaccel);
 
 void
 pointer_notify_motion_absolute(struct libinput_device *device,
 			       uint64_t time,
-			       double x,
-			       double y);
+			       const struct device_coords *point);
 
 void
 pointer_notify_button(struct libinput_device *device,
@@ -289,26 +314,22 @@ pointer_notify_axis(struct libinput_device *device,
 		    uint64_t time,
 		    uint32_t axes,
 		    enum libinput_pointer_axis_source source,
-		    double x,
-		    double y,
-		    double x_discrete,
-		    double y_discrete);
+		    const struct normalized_coords *delta,
+		    const struct discrete_coords *discrete);
 
 void
 touch_notify_touch_down(struct libinput_device *device,
 			uint64_t time,
 			int32_t slot,
 			int32_t seat_slot,
-			double x,
-			double y);
+			const struct device_coords *point);
 
 void
 touch_notify_touch_motion(struct libinput_device *device,
 			  uint64_t time,
 			  int32_t slot,
 			  int32_t seat_slot,
-			  double x,
-			  double y);
+			  const struct device_coords *point);
 
 void
 touch_notify_touch_up(struct libinput_device *device,
