@@ -3,6 +3,6 @@
 /* This is a build-test only */
 
 int
-main(int argc, char **argv) {
+main(void) {
 	return 0;
 }
